/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global console, window, CustomEvent, tizen, localStorage*/

/**
 * Application pressure model module.
 * It is responsible for obtaining and storing information
 * about device pressure sensor value.
 *
 * @module app.model.pressure
 * @namespace app.model.pressure
 * @memberOf app.model
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineAppModelPressure(app) {
    'use strict';

    /**
     * Sensor type.
     *
     * @private
     * @const {string}
     */
    var SENSOR_TYPE = 'PRESSURE',

        /**
         * Storage reference pressure key.
         *
         * @private
         * @const {string}
         */
        REFERENCE_PRESSURE_KEY = 'reference',

        /**
         * Default pressure.
         *
         * @private
         * @const {number}
         */
        DEFAULT_PRESSURE = 1013.25,

        /**
         * The constant of the barometric formula.
         *
         * P - atmospheric pressure.
         * P0 - atmospheric reference pressure.
         * u - molar mass of Earth's air: 0.0289644 [kg / mol]
         * g - gravitational acceleration: 9.80655 [m / s2]
         * R - universal gas constant for air: 8.31432 [J / mol * K]
         * T - air temperature: 298.15 [K] (25 [Celsius])
         * h - altitude [m]
         *
         * h = -(R * T / g * u) * ln(P / P0)
         *
         * BAROMETRIC_FORMULA_CONSTANT = -(R * T / g * u)
         *
         * h = BAROMETRIC_FORMULA_CONSTANT * ln(P / P0)
         *
         * @private
         * @const {number}
         */
        BAROMETRIC_FORMULA_CONSTANT = -8727,

        /**
         * Pressure model module reference.
         *
         * @private
         * @type {object}
         */
        modelPressure = null,

        /**
         * Reference to the sensor service.
         *
         * @private
         * @type {SensorService}
         */
        sensorService = null,

        /**
         * Reference to the pressure sensor.
         *
         * @private
         * @type {PressureSensor}
         */
        pressureSensor = null,

        /**
         * Current pressure.
         *
         * @private
         * @type {number}
         */
        currentPressure = 0;

    // create namespace for the module
    app.model = app.model || {};
    app.model.pressure = app.model.pressure || {};
    modelPressure = app.model.pressure;

    /**
     * Sets current pressure value.
     *
     * @private
     * @param {object} data
     */
    function setCurrentPressureValue(data) {
        currentPressure = data.pressure;
    }

    /**
     * Performs action when sensor data are obtained.
     *
     * Sets current pressure value.
     *
     * @private
     * @param {object} data
     * @fires model.pressure.current.init
     */
    function onPressureSensorData(data) {
        setCurrentPressureValue(data);
        window.dispatchEvent(new CustomEvent('model.pressure.current.init'));
    }

    /**
     * Performs action on sensor change.
     *
     * Sets current pressure value.
     * Triggers model.pressure.current.change event.
     *
     * @private
     * @param {object} data
     * @fires model.pressure.current.change
     */
    function onSensorChange(data) {
        setCurrentPressureValue(data);
        window.dispatchEvent(new CustomEvent('model.pressure.current.change'));
    }

    /**
     * Sets reference pressure value.
     *
     * Saves the reference pressure value into the local storage.
     * Triggers model.pressure.reference.change event.
     *
     * @memberOf app.model.pressure
     * @public
     * @param {number} value
     * @fires model.pressure.reference.change
     */
    modelPressure.setReferencePressure = function setReferencePressure(value) {
        localStorage.setItem(REFERENCE_PRESSURE_KEY, value);
        window.dispatchEvent(
            new CustomEvent('model.pressure.reference.change')
        );
    };

    /**
     * Returns true if sensor is available, false otherwise.
     *
     * @memberOf app.model.pressure
     * @public
     * @returns {boolean}
     */
    modelPressure.isAvailable = function isAvailable() {
        return !!pressureSensor;
    };

    /**
     * Sets sensor change listener.
     *
     * @memberOf app.model.pressure
     * @public
     */
    modelPressure.setChangeListener = function setChangeListener() {
        pressureSensor.setChangeListener(onSensorChange);
    };

    /**
     * Obtains current pressure value.
     *
     * @memberOf app.model.pressure
     * @public
     */
    modelPressure.initCurrentPressure = function initCurrentPressure() {
        pressureSensor.getPressureSensorData(onPressureSensorData);
    };

    /**
     * Returns current pressure value.
     *
     * @memberOf app.model.pressure
     * @public
     * @returns {number}
     */
    modelPressure.getCurrentPressure = function getCurrentPressure() {
        return currentPressure;
    };

    /**
     * Returns reference pressure value.
     *
     * @memberOf app.model.pressure
     * @public
     * @returns {number}
     */
    modelPressure.getReferencePressure = function getReferencePressure() {
        return parseFloat(localStorage.getItem(REFERENCE_PRESSURE_KEY));
    };

    /**
     * Returns altitude value based on the reference and current pressure value.
     *
     * @memberOf app.model.pressure
     * @public
     * @returns {number}
     */
    modelPressure.getAltitude = function getAltitude() {
        return Math.round(
            BAROMETRIC_FORMULA_CONSTANT *
            Math.log(currentPressure / modelPressure.getReferencePressure())
        );
    };

    /**
     * Performs action on start sensor success.
     *
     * Triggers model.pressure.start event.
     *
     * @private
     */
    function onSensorStartSuccess() {
        window.dispatchEvent(new CustomEvent('model.pressure.start'));
    }

    /**
     * Performs action on start sensor error.
     *
     * Logs an error to the console.
     *
     * @private
     * @param {Error} e
     */
    function onSensorStartError(e) {
        console.error('Pressure sensor start error: ', e);
    }

    /**
     * Starts sensor.
     *
     * @memberOf app.model.pressure
     * @public
     */
    modelPressure.start = function start() {
        pressureSensor.start(onSensorStartSuccess, onSensorStartError);
    };

    /**
     * Initializes the pressure model module.
     *
     * @memberOf app.model.pressure
     * @public
     */
    modelPressure.init = function init() {
        sensorService = tizen.sensorservice ||
            (window.webapis && window.webapis.sensorservice) || null;

        if (!sensorService) {
            console.error('Sensor not avalilable.');
        } else {
            try {
                pressureSensor = sensorService.getDefaultSensor(SENSOR_TYPE);
                if (!modelPressure.getReferencePressure()) {
                    modelPressure.setReferencePressure(DEFAULT_PRESSURE);
                }
            } catch (error) {
                if (error.type === 'NotSupportedError') {
                    console.error('Sensor not supported.');
                } else {
                    console.error('Unknown error.');
                }
            }
        }
    };

})(window.app);
