/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global window, document, setTimeout*/

/**
 * Application UI module.
 * It is responsible for managing user interface.
 *
 * @module app.ui
 * @requires {@link app.ui.canvas}
 * @requires {@link app.model.pressure}
 * @namespace app.ui
 * @memberOf app
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineAppUi(app) {
    'use strict';

    /**
     * Calibration message.
     *
     * @private
     * @const {string}
     */
    var CALIBRATION_MSG = 'App will be calibrated.',

        /**
         * UI module reference.
         *
         * @private
         * @type {object}
         */
        ui = null,

        /**
         * UI canvas module reference.
         *
         * @private
         * @type {object}
         */
        uiCanvas = app.ui.canvas,

        /**
         * Pressure model module reference.
         *
         * @private
         * @type {object}
         */
        modelPressure = app.model.pressure,

        /**
         * Calibration page.
         *
         * @private
         * @type {HTMLElement}
         */
        calibrationPage = null,

        /**
         * Calibration page message.
         *
         * @private
         * @type {HTMLElement}
         */
        calibrationPageMessage = null,

        /**
         * Calibration no button.
         *
         * @private
         * @type {HTMLElement}
         */
        calibrationNoBtn = null,

        /**
         * Calibration yes button.
         *
         * @private
         * @type {HTMLElement}
         */
        calibrationYesBtn = null,

        /**
         * Information popup.
         *
         * @private
         * @type {HTMLElement}
         */
        informationPopup = null,

        /**
         * Information button.
         *
         * @private
         * @type {HTMLElement}
         */
        informationBtn = null,

        /**
         * Menu calibration button.
         *
         * @private
         * @type {HTMLElement}
         */
        menuCalibrationBtn = null,

        /**
         * Reference pressure value element.
         *
         * @private
         * @type {HTMLElement}
         */
        referencePressureValue = null,

        /**
         * Current pressure value element.
         *
         * @private
         * @type {HTMLElement}
         */
        currentPressureValue = null,

        /**
         * Indicates whether UI is initialized.
         *
         * @private
         * @type {boolean}
         */
        isUIInitialized = false;

    // create namespace for the module
    app.ui = app.ui || {};
    ui = app.ui;

    /**
     * Updates value of the current pressure UI element.
     *
     * @private
     */
    function updateCurrentPressure() {
        currentPressureValue.innerText =
            modelPressure.getCurrentPressure().toFixed(2);
    }

    /**
     * Updates value of the reference pressure UI element.
     *
     * @private
     */
    function updateReferencePressure() {
        referencePressureValue.innerText =
            modelPressure.getReferencePressure().toFixed(2);
    }

    /**
     * Handles model.pressure.current.change event.
     *
     * Updates value of the current pressure UI element.
     *
     * @private
     */
    function onPressureCurrentChange() {
        updateCurrentPressure();
    }

    /**
     * Handles model.pressure.reference.change event.
     *
     * Updates value of the reference pressure UI element.
     *
     * @private
     */
    function onPressureReferenceChange() {
        updateReferencePressure();
    }

    /**
     * Handles model.pressure.start event.
     *
     * Initializes current pressure value.
     * Updates current and reference pressure UI elements.
     *
     * @private
     */
    function onPressureStart() {
        modelPressure.initCurrentPressure();
        updateCurrentPressure();
        updateReferencePressure();
    }

    /**
     * Handles model.pressure.current.init event.
     *
     * Initializes canvas module.
     *
     * @private
     */
    function onPressureCurrentInit() {
        uiCanvas.init();
    }

    /**
     * Hides information popup.
     *
     * @private
     */
    function hideInformationPopup() {
        informationPopup.classList.add('hidden');
    }

    /**
     * Hides calibration page.
     *
     * @private
     */
    function hideCalibrationPage() {
        calibrationPage.classList.add('hidden');
    }

    /**
     * Handles click event on calibration NO button.
     *
     * Hides calibration page.
     *
     * @private
     */
    function onCalibrationNoBtnClick() {
        hideCalibrationPage();
        initUI();
    }

    /**
     * Handles touchstart event on calibration NO button.
     *
     * Adds pressed CSS class to the calibration NO button element.
     *
     * @private
     */
    function onCalibrationNoBtnTouchStart() {
        calibrationNoBtn.classList.add('pressed');
    }

    /**
     * Handles touchend event on calibration NO button.
     *
     * Removes pressed CSS class from the calibration NO button element.
     *
     * @private
     */
    function onCalibrationNoBtnTouchEnd() {
        calibrationNoBtn.classList.remove('pressed');
    }

    /**
     * Initializes UI.
     */
    function initUI() {
        if (isUIInitialized) {
            return;
        }

        calibrationPageMessage.innerText = CALIBRATION_MSG;
        calibrationPageMessage.classList.add('bold');
        setTimeout(hideInformationPopup, 5000);
        isUIInitialized = true;
    }

    /**
     * Handles click event on calibration YES button.
     *
     * Saves reference pressure.
     * Hides calibration page.
     * If necessary, starts canvas animation.
     *
     * @private
     */
    function onCalibrationYesBtnClick() {
        modelPressure.setReferencePressure(modelPressure.getCurrentPressure());
        hideCalibrationPage();
        initUI();
        uiCanvas.checkAltitude();
    }

    /**
     * Handles touchstart event on calibration YES button.
     *
     * Adds pressed CSS class to the calibration YES button element.
     *
     * @private
     */
    function onCalibrationYesBtnTouchStart() {
        calibrationYesBtn.classList.add('pressed');
    }

    /**
     * Handles touchend event on calibration YES button.
     *
     * Removes pressed CSS class from the calibration YES button element.
     *
     * @private
     */
    function onCalibrationYesBtnTouchEnd() {
        calibrationYesBtn.classList.remove('pressed');
    }

    /**
     * Handles click event on information OK button.
     *
     * Hides information popup.
     *
     * @private
     */
    function onInformationBtnClick() {
        hideInformationPopup();
    }

    /**
     * Handles touchstart event on information OK button.
     *
     * Adds pressed CSS class to the information OK button element.
     *
     * @private
     */
    function onInformationBtnTouchStart() {
        informationBtn.classList.add('pressed');
    }

    /**
     * Handles touchend event on information OK button.
     *
     * Removes pressed CSS class from the information OK button element.
     *
     * @private
     */
    function onInformationBtnTouchEnd() {
        informationBtn.classList.remove('pressed');
    }

    /**
     * Handles click event on menu calibration button.
     *
     * Shows the calibration page.
     *
     * @private
     */
    function onMenuCalibrationBtnClick() {
        calibrationPage.classList.remove('hidden');
    }

    /**
     * Registers event listeners.
     *
     * @private
     */
    function bindEvents() {
        window.addEventListener(
            'model.pressure.start',
            onPressureStart
        );
        window.addEventListener(
            'model.pressure.current.init',
            onPressureCurrentInit
        );
        window.addEventListener(
            'model.pressure.current.change',
            onPressureCurrentChange
        );
        window.addEventListener(
            'model.pressure.reference.change',
            onPressureReferenceChange
        );
        calibrationNoBtn.addEventListener('click', onCalibrationNoBtnClick);
        calibrationNoBtn.addEventListener(
            'touchstart',
            onCalibrationNoBtnTouchStart
        );
        calibrationNoBtn.addEventListener(
            'touchend',
            onCalibrationNoBtnTouchEnd
        );
        calibrationYesBtn.addEventListener('click', onCalibrationYesBtnClick);
        calibrationYesBtn.addEventListener(
            'touchstart',
            onCalibrationYesBtnTouchStart
        );
        calibrationYesBtn.addEventListener(
            'touchend',
            onCalibrationYesBtnTouchEnd
        );
        informationBtn.addEventListener('click', onInformationBtnClick);
        informationBtn.addEventListener(
            'touchstart',
            onInformationBtnTouchStart
        );
        informationBtn.addEventListener(
            'touchend',
            onInformationBtnTouchEnd
        );
        menuCalibrationBtn.addEventListener('click', onMenuCalibrationBtnClick);
    }

    /**
     * Initializes the UI module.
     *
     * @memberOf app.ui
     * @public
     */
    ui.init = function init() {
        calibrationPage = document.getElementById('calibration');
        calibrationPageMessage = calibrationPage.querySelector('.message');
        calibrationNoBtn = document.getElementById('calibration-no-btn');
        calibrationYesBtn = document.getElementById('calibration-yes-btn');
        informationPopup = document.getElementById('information');
        informationBtn = document.getElementById('information-btn');
        menuCalibrationBtn = document.getElementById('menu-calibration-btn');
        referencePressureValue = document.getElementById(
            'reference-pressure-value'
        );
        currentPressureValue = document.getElementById(
            'current-pressure-value'
        );

        bindEvents();

        modelPressure.init();
        if (modelPressure.isAvailable()) {
            modelPressure.setChangeListener();
            modelPressure.start();
        }
    };

})(window.app);
